﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using System.Text.RegularExpressions;
using System.Resources;
using System.Collections;
using System.ComponentModel.Design;


namespace EduNetworkBuilder
{
  

    public partial class TranslationWindow : Form
    {
        Network myNet = null;
        string[,] LanguageChoices = { { NB.Translate("NB_NBEn"), "en" }, { NB.Translate("NB_NBFr"), "fr" } };
        string FormChosenDir = "";
        List<TranslationResxFile> TranslationResxFiles = new List<TranslationResxFile>();
        string EnglishTargetFile = "edustrings.resx";
        bool ChangedFormData = false;


        public TranslationWindow(Network ToEdit)
        {
            InitializeComponent();
            this.KeyPreview = true;
            myNet = ToEdit;

            LocalSetup();
        }

        void LocalSetup()
        {            
            cb1Language.Items.Clear();
            cb2Language.Items.Clear();

            cbFormLang1Choice.Items.Clear();
            cbFormLang2Choice.Items.Clear();

            cbFormHighPriority.CheckedChanged += PopulateFormTranslationItems;
            cbFormShowTranslated.CheckedChanged += PopulateFormTranslationItems;

            panelTranslateFormItems.Enabled = false;

            NBSettings mySettings = NB.GetSettings();

            //See if the file we are editing has the right folder.  If so, we do not need to choose one
            FormChosenDir = mySettings.FormLastTransDir;
            if (!TryToChooseFormDir(FormChosenDir))
                TryToChooseFormDir(Path.GetDirectoryName(myNet.NetworkFilename));

            for (int i = 0; i < LanguageChoices.GetLength(0); i++)
            {
                cb1Language.Items.Add(LanguageChoices[i, 0]);
                cb2Language.Items.Add(LanguageChoices[i, 0]);
                cbFormLang1Choice.Items.Add(LanguageChoices[i, 0]);
                cbFormLang2Choice.Items.Add(LanguageChoices[i, 0]);

                if (mySettings.LastTrans1Lang == LanguageChoices[i, 1])
                {
                    cb1Language.SelectedItem = LanguageChoices[i, 0];
                    cbFormLang1Choice.SelectedItem = LanguageChoices[i, 0];
                }
                if (mySettings.LastTrans2Lang == LanguageChoices[i, 1])
                {
                    cb2Language.SelectedItem = LanguageChoices[i, 0];
                    cbFormLang2Choice.SelectedItem = LanguageChoices[i, 0];
                }
            }
            if (cb1Language.SelectedIndex == -1) cb1Language.SelectedIndex = 0;
            if (cb2Language.SelectedIndex == -1) cb2Language.SelectedIndex = 0;
            Icon = Properties.Resources.NBIco;
            lblStatus.Text = "";
            LanguagifyComponents();
        }

        private void LanguagifyComponents()
        {
            Text = NB.Translate("TW_Translate");
            btnSave.Text = NB.Translate("_Save");
            btnClose.Text = NB.Translate("_Done");
            tpMessageTitle.Text = NB.Translate("TW_Message");
            tpFormStuff.Text = NB.Translate("TW_Form");
        }

        private void ClosingStuff()
        {
            NBSettings mySettings = NB.GetSettings();
            for (int i = 0; i < LanguageChoices.GetLength(0); i++)
            {
                if (cb1Language.Text == LanguageChoices[i, 0])
                {
                    mySettings.LastTrans1Lang = LanguageChoices[i, 1];
                }
                if (cb2Language.Text == LanguageChoices[i, 0])
                {
                    mySettings.LastTrans2Lang = LanguageChoices[i, 1];
                }
                mySettings.FormLastTransDir = FormChosenDir;
            }
            mySettings.Save();
        }

        string CodeFromLang(string language)
        {
            for (int i = 0; i < LanguageChoices.GetLength(0); i++)
            {
                if (language == LanguageChoices[i, 0]) return LanguageChoices[i, 1];
            }
            return "";
        }

        string LangFromCode(string Code)
        {
            for (int i = 0; i < LanguageChoices.GetLength(0); i++)
            {
                if (Code == LanguageChoices[i, 1]) return LanguageChoices[i, 0];
            }
            return "";
        }

        private void btnClose_Click(object sender, EventArgs e)
        {            
            Close();
        }

        private void Save()
        {
            if (File.Exists(myNet.NetworkFilename))
            {
                myNet.Save();
                //Console.WriteLine("Saved");
                lblStatus.Text = NB.Translate("_Saved");
            }
            if(ChangedFormData)
            {
                foreach (TranslationResxFile one in TranslationResxFiles)
                    one.Save();
                ChangedFormData = false;
                lblStatus.Text = NB.Translate("_Saved");
            }
        }

        private void TranslationWindow_KeyDown(object sender, KeyEventArgs e)
        {
            if(e.Modifiers == Keys.Control && e.KeyCode == Keys.S)
            {
                //ctrl-s was pressed
                Save();
            }
        }

        private void TranslationWindow_FormClosing(object sender, FormClosingEventArgs e)
        {
            ClosingStuff();
        }

        private void cb1Language_SelectedIndexChanged(object sender, EventArgs e)
        {
            string What = CodeFromLang(cb1Language.Text);
            if (What != "")
            {
                tb1Message.Text = myNet.NetMessage.GetText(What);
                tb1Title.Text = myNet.NetTitle.GetText(What);
                tb1URL.Text = myNet.NetURL.GetText(What);
            }
        }

        private void cb2Language_SelectedIndexChanged(object sender, EventArgs e)
        {
            string What = CodeFromLang(cb2Language.Text);
            if (What != "")
            {
                tb2Message.Text = myNet.NetMessage.GetText(What);
                tb2Title.Text = myNet.NetTitle.GetText(What);
                tb2URL.Text = myNet.NetURL.GetText(What);
            }
        }

        void RefreshContents()
        {
            cb1Language_SelectedIndexChanged(null, EventArgs.Empty);
            cb2Language_SelectedIndexChanged(null, EventArgs.Empty);
            lblStatus.Text = "";
        }

        private void tb1Message_Leave(object sender, EventArgs e)
        {
            string What = CodeFromLang(cb1Language.Text);
            if (What != "")
            {
                myNet.NetMessage.ChangeText(What,tb1Message.Text);
            }
            RefreshContents();
        }

        private void tb2Message_Leave(object sender, EventArgs e)
        {
            string What = CodeFromLang(cb2Language.Text);
            if (What != "")
            {
                myNet.NetMessage.ChangeText(What, tb2Message.Text);
            }
            RefreshContents();
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            Save();
        }

        private void tb1Title_Leave(object sender, EventArgs e)
        {
            string What = CodeFromLang(cb1Language.Text);
            if (What != "")
            {
                myNet.NetTitle.ChangeText(What, tb1Title.Text);
            }
            RefreshContents();
        }

        private void tb2Title_Leave(object sender, EventArgs e)
        {
            string What = CodeFromLang(cb2Language.Text);
            if (What != "")
            {
                myNet.NetTitle.ChangeText(What, tb2Title.Text);
            }
            RefreshContents();
        }

        private void tb1URL_Leave(object sender, EventArgs e)
        {
            string What = CodeFromLang(cb1Language.Text);
            if (What != "")
            {
                myNet.NetURL.ChangeText(What, tb1URL.Text);
            }
            RefreshContents();
        }

        private void tb2URL_Leave(object sender, EventArgs e)
        {
            string What = CodeFromLang(cb2Language.Text);
            if (What != "")
            {
                myNet.NetURL.ChangeText(What, tb2URL.Text);
            }
            RefreshContents();
        }




        //**********************************************************************//
        //******************* Form Translation Things **************************//
        //**********************************************************************//
        bool TryToChooseFormDir(string Choice)
        {
            if (Choice == null || Choice == "") return false;
            //We want to have a file named edustrings.resx there

            //There may be other files: edustrings.fr.resx
            //If these exist, we are OK.  Otherwise, see if there is a "languages" dir and try that if it does exist
            string toCheck = Path.GetFileName(Choice).ToLower();
            if (File.Exists(Choice))
                if (toCheck == EnglishTargetFile)
                {
                    FormChosenDir = Path.GetDirectoryName(Choice);
                    LoadResxFiles();
                    panelTranslateFormItems.Enabled = true;
                    lblFormDir.Text = "Dir: " + FormChosenDir;
                    lblFormDir.AutoSize = true;
                    lblFormDir.MaximumSize = new Size(250, 200);
                    return true;
                }
                
            if(Directory.Exists(Choice))
            {
                //It is a directory.  See if it has the correct file in it.
                if (TryToChooseFormDir(Path.Combine(Choice, EnglishTargetFile)))
                {
                    return true; //It was successful
                }
                //If we get here, it did not have the file.  See if there is a "languages" directory
                if (TryToChooseFormDir(Path.Combine(Choice, "languages", EnglishTargetFile))) 
                {
                    return true; //It was successful
                }
            }
            return false;
        }

        void LoadResxFiles()
        {
            if (!Directory.Exists(FormChosenDir)) return;
            var resxFiles = Directory.EnumerateFiles(FormChosenDir, "*.resx", SearchOption.AllDirectories);
            TranslationResxFiles.Clear();

            foreach (string currentFile in resxFiles)
            {
                TranslationResxFile TRF = new TranslationResxFile(currentFile);
                TranslationResxFiles.Add(TRF);
            }
            PopulateFormTranslationItems();
        }

        void PopulateFormTranslationItems(object sender, EventArgs e)
        {
            PopulateFormTranslationItems();
        }

        private void PopulateFormTranslationItems()
        {
            cbFormItemChoice.Items.Clear();
            TranslationResxFile en = FileFromLangCode("en");
            List<string> Priorities = new List<string>() { "_", "NB_", "DC_" };
            if(en != null)
            {
                foreach(TranslationItem TI in en.Items)
                {
                    bool AddTranslated = false;
                    bool AddHigh = false;
                    if (!TI.translated || cbFormShowTranslated.Checked)
                        AddTranslated = true;
                    if (cbFormHighPriority.Checked)
                    {
                        foreach(string one in Priorities)
                        {
                            if(TI.Key.StartsWith(one))
                            {
                                AddHigh = true;
                                break;
                            }
                        }
                    }
                    else
                        AddHigh = true;
                    if(AddTranslated && AddHigh)
                        cbFormItemChoice.Items.Add(TI.Key);
                }                
            }
        }

        private TranslationResxFile FileFromLangCode(string code)
        {
            foreach (TranslationResxFile one in TranslationResxFiles)
                if (one.language == code) return one;
            return null;
        }

        private TranslationItem ItemFromKey(TranslationResxFile File, string Key)
        {
            if (File == null) return null;
            foreach(TranslationItem TI in File.Items)
            {
                if (TI.Key == Key)
                    return TI;
            }
            return null;
        }

        private TranslationItem GetEditedTranslationItem()
        {
            string code = CodeFromLang(cbFormLang1Choice.Text);
            TranslationResxFile TRf = FileFromLangCode(code);
            TranslationItem TI = ItemFromKey(TRf, cbFormItemChoice.Text);
            return TI;
    }

    private void btnChooseDir_Click(object sender, EventArgs e)
        {
            string startingDir = FormChosenDir;
            if (FormChosenDir == "" && File.Exists(myNet.NetworkFilename))
                startingDir = Path.GetDirectoryName(myNet.NetworkFilename);
            //Prompt for a directory.

            OpenFileDialog ofd = NB.SelectDirectoryDialog("Directory with language files", startingDir);
            if(ofd.FileName != "")
            {
                //try to see if the chosen filename has appropriate files.  We need the English, at least
                if(!TryToChooseFormDir(Path.GetDirectoryName(ofd.FileName)))
                {
                    MessageBox.Show("Invalid Directory.");
                }
            }
        }

        private void cbFormItemChoice_SelectedIndexChanged(object sender, EventArgs e)
        {
            string code = CodeFromLang(cbFormLang1Choice.Text);
            TranslationResxFile TRf = FileFromLangCode(code);
            TranslationItem TI = ItemFromKey(TRf, cbFormItemChoice.Text);
            if(TRf != null && TI != null)
            {
                tbFormItem1.Text = TI.Value;
            }

             code = CodeFromLang(cbFormLang2Choice.Text);
             TRf = FileFromLangCode(code);
             TI = ItemFromKey(TRf, cbFormItemChoice.Text);
            if (TRf != null && TI != null)
            {
                tbFormItem2.Text = TI.Value;
                cbFormItemTranslated.Checked = TI.translated;
            }

        }

        private void cbFormItemTranslated_CheckedChanged(object sender, EventArgs e)
        {
            TranslationItem TI = GetEditedTranslationItem();
            if(TI != null)
            {
                TI.translated = cbFormItemTranslated.Checked;
                ChangedFormData = true;
            }
        }
    }

    #region TranslationClasses
    public class TranslationItem
    {
        public string Key = "";
        public string Value = "";
        public string Comment = "";
        public bool translated = false;
    }

    public class TranslationResxFile
    {
        public string filename;
        public List<TranslationItem> Items = new List<TranslationItem>();
        public string language = "";
        string beginning = "translated: ";

        public TranslationResxFile(string File)
        {
            filename = File;
            Load();
            string fn = Path.GetFileName(File);
            if (fn == "edustrings.resx")
            {
                language = "en";
            }
            else
                language = fn.Substring(11, 2);
        }

        void Load()
        {
            if(! File.Exists(filename))
            {
                //The file does not exist yet.  Nothing to load.  Exit gracefully
                return;
            }
            Items.Clear();
            ResXResourceReader InResx = new ResXResourceReader(filename);
            InResx.UseResXDataNodes = true;
            System.Reflection.Assembly currentAssembly = System.Reflection.Assembly.GetExecutingAssembly();

            foreach (DictionaryEntry d in InResx)
            {
                ResXDataNode rdn = (ResXDataNode)d.Value;
                TranslationItem TI = new TranslationItem();
                TI.Comment = rdn.Comment;
                if (TI.Comment.StartsWith(beginning))
                {
                    //trim off the beginning "translated" tag
                    TI.Comment = rdn.Comment.Substring(0, beginning.Length - 1);
                    TI.translated = true;
                }
                TI.Key = rdn.Name;
                TI.Value = (string)rdn.GetValue(new System.Reflection.AssemblyName[] { currentAssembly.GetName() });
                Items.Add(TI);
            }
            Items.Sort((a,b) => a.Key.CompareTo(b.Key));
            InResx.Close();
        }

        public void Save()
        {
            string tmpfile = Path.Combine(Path.GetDirectoryName(filename), Path.GetFileNameWithoutExtension(filename) + "-tmp" + Path.GetExtension(filename));
            ResXResourceWriter WriteResx = new ResXResourceWriter(tmpfile);

            foreach (TranslationItem d in Items)
            {
                ResXDataNode rdn = new ResXDataNode(d.Key, d.Value);
                if (d.translated)
                    rdn.Comment = beginning + d.Comment;
                else
                    rdn.Comment = d.Comment;                
                WriteResx.AddResource(rdn);
                //Console.WriteLine("Adding " + toWrite.Name);
            }
            WriteResx.Generate();
            WriteResx.Close();
            //If we have an error, we should have exited and gone to "catch."
            //Proceed to put the new file
            File.Delete(filename);
            File.Move(tmpfile, filename);
        }
    }
    #endregion
}
